/**
 * gl-date-picker - v0.3.2 - 2017-04-07
 * http://www.glispa.com/
 *
 * Copyright (c) 2017 glispa
 * Licensed MIT <https://raw.github.com/ngbp/ngbp/master/LICENSE>
 */
(function ( window, angular, undefined ) {

"use strict";

angular.module('glDatePickerController', [
  'glDatePickerFactory', 'vendor.jQuery'])
  .controller('glDatePickerController', ['$rootScope', '$scope', '$filter', 'glDatePickerFactory', '$',
    function ($rootScope, $scope, $filter, glDatePickerFactory, $) {

    var monthNames = ["January", "February", "March", "April", "May", "June",
      "July", "August", "September", "October", "November", "December"
    ];
    var weekDays = [{name: "M"}, {name: "T"}, {name: "W"}, {name: "T"}, {name: "F"}, {name: "S"}, {name: "S"}];
    var today = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      today.setHours(0, 0, 0);
    /**
     * @description initialize the calendar loading the preselected dates
     *
     */
    $scope.initDatePicker = function () {
      $scope.startDate = new Date($scope.dates[0]);
      $scope.endDate = new Date($scope.dates[1]);
      $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
      $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
      $scope.calendarOpen = false;
      $scope.selectedDateType = "";
      glDatePickerFactory.checkDefaultDates($scope);
      $scope.dateGrid = {
        headers: weekDays,
        weeks: []
      };
    };

    /**
     * @description opens up the calendar setting the date type to start or end
     *
     * @param {Object} dateType - the selected date type
     *
     */
    $scope.openCalendar = function (dateType) {
      if (!$scope.settings || !$scope.settings.disable) {
        $scope.selectedDateType = dateType;
        if ($scope.selectedDateType === "start") {
          $scope.baseDate = new Date($scope.startDate);
        } else if ($scope.selectedDateType === "end") {
          $scope.baseDate = new Date($scope.endDate);
        }
        $scope.baseDate.setDate(1);
        $scope.showMonth = monthNames[$scope.baseDate.getMonth()];
        $scope.showYear = $scope.baseDate.getFullYear();
        $scope.setDateGrid();
        $scope.calendarOpen = true;
      }
    };

    /**
     * @description loads the current page of the calendar
     *
     */
    $scope.setDateGrid = function () {
      $scope.dateGrid.weeks = glDatePickerFactory.setDateGrid($scope.baseDate, $scope.startDate, $scope.endDate, $scope.selectedDateType);
    };

    /**
     * @description sets the selected date as start or end date
     *
     * @param {Object} date - the selected date type
     *
     */
    $scope.selectDate = function (date) {
      if (date.isNotThisMonth === false) {
        $scope.selectedPreset = null;

        if ((glDatePickerFactory.compareDates(date.date, $scope.startDate) < 0) || ($scope.selectedDateType === "start" &&
          glDatePickerFactory.compareDates(date.date, $scope.endDate) <= 0 )) {
          $scope.startDate = date.date;
          $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
          $scope.selectedDateType = "end";
          $('#endInput').focus();
        } else if ((glDatePickerFactory.compareDates(date.date, $scope.endDate) > 0) || ($scope.selectedDateType === "end" &&
          glDatePickerFactory.compareDates(date.date, $scope.startDate) >= 0)) {
          $scope.endDate = date.date;
          $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
          $scope.selectedDateType = "start";
          $('#startInput').focus();
        }
        $scope.setDateGrid();
        glDatePickerFactory.checkDefaultDates($scope);
      }
    };

    /**
     * @description changes the selected month to next one
     *
     */
    $scope.nextMonth = function () {
      $scope.baseDate.setMonth($scope.baseDate.getMonth() + 1);
      $scope.setDateGrid();
      $scope.showMonth = monthNames[$scope.baseDate.getMonth()];
      $scope.showYear = $scope.baseDate.getFullYear();
    };

    /**
     * @description changes the selected month to previous one
     *
     */
    $scope.prevMonth = function () {
      $scope.baseDate.setMonth($scope.baseDate.getMonth() - 1);
      $scope.setDateGrid();
      $scope.showMonth = monthNames[$scope.baseDate.getMonth()];
      $scope.showYear = $scope.baseDate.getFullYear();
    };

    /**
     * @description restores last saved start and end date and close the calendar
     *
     */
    $scope.cancel = function () {
      $scope.calendarOpen = false;
      $scope.selectedDateType = "";
      $scope.startDate = new Date($scope.dates[0]);
      $scope.endDate = new Date($scope.dates[1]);
      $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
      $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
      if ($scope.relativeDates) {
        $scope.relativeStartDate = $scope.relativeDates[0];
        $scope.relativeEndDate = $scope.relativeDates[1];
        glDatePickerFactory.checkDefaultDates($scope);
      }
    };

    /**
     * @description set current start and end dates as selected one and closes the menu
     *
     */
    $scope.apply = function () {
      $scope.calendarOpen = false;
      $scope.selectedDateType = "";
      var setDates = {
        dates: [ $filter('date')($scope.startDate, 'yyyy-MM-dd'), $filter('date')($scope.endDate, 'yyyy-MM-dd')]
      };
      if ($scope.selectedPreset) {
        setDates.relativeDates = [$scope.relativeStartDate, $scope.relativeEndDate];
      } else {
        $scope.startDate.setHours(0, 0, 0);
        $scope.endDate.setHours(0, 0, 0);
        $scope.relativeStartDate = 'd-' + Math.ceil(Math.abs($scope.startDate.getTime() - today.getTime()) / (1000 * 3600 * 24));
        $scope.relativeEndDate = 'd-' + Math.ceil(Math.abs($scope.endDate.getTime() - today.getTime()) / (1000 * 3600 * 24));
        setDates.relativeDates = [$scope.relativeStartDate, $scope.relativeEndDate];
      }
      $rootScope.$broadcast('dateSelected', setDates);
    };

    /**
     * @description sets start and end dates according to the selected preset
     *
     * @param {Object} preset - the selected preset
     */
    $scope.selectPreset = function (preset) {
      $scope.selectedPreset = preset;
      $scope.startDate = glDatePickerFactory.getDateFromRelative(preset.startDate);
      $scope.endDate = glDatePickerFactory.getDateFromRelative(preset.endDate);
      $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
      $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
      $scope.relativeStartDate = preset.startDate;
      $scope.relativeEndDate = preset.endDate;

      $scope.openCalendar($scope.selectedDateType);
      glDatePickerFactory.checkDefaultDates($scope);
    };

    /**
     * @description checks if input dates are valid, if so sets start or end date otherwise rest input to default
     *
     * @param {string} dateType - start or end for start or end date
     */
    $scope.testDate = function (dateType) {
      var date = null;
      if (dateType === 'start') {
        date = new Date($scope.startDateDescription);
        if (date && !isNaN(date) && glDatePickerFactory.compareDates(date, $scope.endDate) <= 0
          && glDatePickerFactory.compareDates(date, $scope.startDate) !== 0) {
          $scope.startDate = new Date(date);
          $scope.openCalendar("end");
          $('#endInput').focus();
        } else {
          $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
        }
      } else if (dateType === 'end') {
        date = new Date($scope.endDateDescription);
        if (date && !isNaN(date) && glDatePickerFactory.compareDates(date, $scope.startDate) >= 0
          && glDatePickerFactory.compareDates(date, $scope.endDate) !== 0) {
          $scope.endDate = new Date(date);
          $scope.openCalendar("start");
          $('#startInput').focus();
        } else {
          $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
        }
      }
    };

    $scope.initDatePicker();

    $scope.$watchCollection('dates', function () {
      $scope.initDatePicker();
    });
}]);

"use strict";

angular.module('glDatePicker', ['glDatePickerController'])
  .directive('glDatePicker', function () {
    return {
      restrict: 'EA',
      templateUrl: 'glDatePicker/glDatePicker.tpl.html',
      controller: 'glDatePickerController',
      scope: {
        settings: '=',
        dates: '=',
        relativeDates: '='
      }
    };
  });

"use strict";
angular.module ('glDatePickerFactory', [])
  .factory ('glDatePickerFactory', ['$rootScope', '$filter',
  function ($rootScope, $filter) {
    return {
      /**
       * @description checks if selected start and end date correspond to the default values and set the scope.defaultDates
       * accordingly
       *
       * @param {Object} scope - the controller scope
       */
      checkDefaultDates: function (scope) {
        if (scope.settings && scope.settings.defaults) {
          var defaultStartDate = this.getDateFromRelative(scope.settings.defaults[0]);
          var defaultEndDate = this.getDateFromRelative(scope.settings.defaults[1]);
          if (!defaultStartDate || !defaultEndDate) {
            scope.defaultDates = true;
          } else {
            if (this.compareDates(scope.startDate, defaultStartDate) === 0 &&
              this.compareDates(scope.endDate, defaultEndDate) === 0) {
              scope.defaultDates = true;
            } else {
              scope.defaultDates = false;
            }
          }
        } else {
          scope.defaultDates = true;
        }
      },

      /**
       * @description it creates the object representing the page of the calendar to be shown
       *
       * @param {Object} baseDate - baseDate is set to the 1st of the currently selected month
       * @param {Object} startDate - the selected start date
       * @param {Object} endDate - the selected end date
       * @param {Object} selectedDateType - the type of that is being selected: can be start date or end date
       *
       * @returns {Object} weeks the object representing the calendar days to show
       */
      setDateGrid: function (baseDate, startDate, endDate, selectedDateType) {
        var days = this.getDaysToShow(baseDate, startDate, endDate, selectedDateType);
        var weeks = [];
        for (var w = 0; w < 6; w++ ) {
          var week = {days: []};
          for (var d = 0; d < 7; d++ ) {
            var dayElement = days[w * 7 + d];
            if (dayElement.isSelectedDate) {
              dayElement.class = "currentlySelectedDate";
            } else if (dayElement.isInPeriod) {
              dayElement.class = "currentlySelectedPeriod";
            } else if (dayElement.isToday) {
              dayElement.class = "today";
            } else if (dayElement.isNotThisMonth) {
              dayElement.class = "notThisMonth";
            }
            week.days.push(days[w * 7 + d]);
          }
          weeks.push(week);
        }
        return weeks;
      },

      /**
       * @description gets the days in the selected month and year
       *
       * @param {Object} baseDate - baseDate is set to the 1st of the currently selected month
       * @param {Object} startDate - the selected start date
       * @param {Object} endDate - the selected end date
       * @param {Object} selectedDateType - the type of that is being selected: can be start date or end date
       *
       * @returns {Object} days the list of days
       */
      getDaysToShow: function (baseDate, startDate, endDate, selectedDateType) {
        var days = [];
        var firstDay = baseDate.getDay() - 1;
        firstDay = (firstDay < 0) ? 6 : firstDay;
        for (var i = 0; i < firstDay; i ++) {
          var pastDate = new Date(baseDate.getFullYear(), baseDate.getMonth(), baseDate.getDate() - (firstDay - i));
          var inPeriod = false;
          if (this.compareDates(pastDate, startDate) >= 0 && this.compareDates(pastDate, endDate) <= 0) {
            inPeriod = true;
          }
          days.push({date: pastDate, name: pastDate.getDate(), isInPeriod: inPeriod, isNotThisMonth: true});
        }

        for (var j = 0; j < 42 - firstDay; j ++) {
          var date = new Date(baseDate.getFullYear(), baseDate.getMonth(), baseDate.getDate() + j);
          var dateElement = {date: date, name: date.getDate()};
          dateElement.isNotThisMonth = (date.getMonth() !== baseDate.getMonth());
          dateElement.isSelectedDate = false;


          dateElement.isInPeriod = this.compareDates(date, startDate) >= 0 && this.compareDates(date, endDate) <= 0;
          if (selectedDateType === "start" && this.compareDates(date, startDate) === 0 ||
            selectedDateType === "end" && this.compareDates(date, endDate) === 0 ) {
            dateElement.isSelectedDate = true;
          }
          if (this.compareDates(date, new Date()) === 0) {
            dateElement.isToday = true;
          }
          days.push(dateElement);
        }

        return days;
      },

      /**
       * @description transform relative date in absolute date, based on today date
       *
       * @param {Object} relativeDate - the relative start date
       * @param {Object} endRelative - the relative end date
       *
       * @returns {Object} object containing absolute dates
       */
      getDateFromRelative: function (relativeDate) {
        if (!relativeDate){
          return null;
        }
        var type = relativeDate.split('-')[0];
        var value = relativeDate.split('-')[1];
        var date = new Date();
        switch (type) {
          case 'd':
            date.setDate(date.getDate() - value);
            break;
          case 'w':
            var mon = (date.getDay() === 0) ? 6 : date.getDay() % 7 - 1 ;
            date.setDate(date.getDate() - (mon + 7 * value));
            break;
          case 'we':
            var sun = (date.getDay() === 0) ? 0 : 7 - date.getDay();
            date.setDate(date.getDate() - (7 * value) + sun);
            break;
          case 'm':
            date.setDate(1);
            date.setMonth(date.getMonth() - value);
            break;
          case 'me':
            date.setMonth(date.getMonth() - value + 1);
            date.setDate(0);
            break;
          case 'y':
            date.setDate(1);
            date.setMonth(0);
            date.setFullYear(date.getFullYear() - value);
            break;
          case 'ye':
            date.setMonth(0);
            date.setFullYear(date.getFullYear() - value + 1);
            date.setDate(0);
            break;
        }

        date.setHours(0, 0, 0);
        return date;
      },

      /**
       * @description check the two dates (not caring about hours) and returns -1 id the first is smaller, 0 if equals
       * and 1 if bigger
       *
       * @param {Object} date1 - the first date
       * @param {Object} date2 - the second date
       *
       * @returns {Number} integer -1, 0, 1
       */
      compareDates: function (date1, date2) {
        if ($filter('date')(date1, 'yyyy-MM-dd') === $filter('date')(date2, 'yyyy-MM-dd')) {
          return 0;
        } else if (date1 > date2) {
          return 1;
        } else {
          return -1;
        }
      }
    };
  }]);
"use strict";

angular.module('vendor.jQuery', []).
  factory('$', function () {
    /**
     * @description assumes jQuery has already been loaded on the page
     */
    return window.$;
  }
);
angular.module('glDatePicker').run(['$templateCache', function($templateCache) {
  'use strict';

  $templateCache.put('glDatePicker/glDatePicker.tpl.html',
    "<div class=\"selectedDate\" ng-class=\"{true:'', false:'notDefaultDates'}[defaultDates]\">\n" +
    "    <div class=\"calendar\">\n" +
    "        <div class=\"CalendarIcon\"></div>\n" +
    "    </div>\n" +
    "\n" +
    "    <div class=\"startDate\" ng-click=\"openCalendar('start')\" >\n" +
    "        <input id=\"startInput\" ng-blur=\"testDate('start')\" ng-disabled=\"settings.disable\" ng-keypress=\"($event.which === 13)?testDate('start'):0\"\n" +
    "               ng-model=\"startDateDescription\" class=\"dateInput\"\n" +
    "               ng-class=\"{true: 'selectedDateType', false: ''}[selectedDateType === 'start']\"/>\n" +
    "    </div>\n" +
    "    <div class=\"separator\"> -</div>\n" +
    "    <div class=\"endDate\" ng-click=\"openCalendar('end')\">\n" +
    "        <input id=\"endInput\" ng-blur=\"testDate('end')\" ng-disabled=\"settings.disable\" ng-keypress=\"($event.which === 13)?testDate('end'):0\"\n" +
    "               ng-model=\"endDateDescription\" class=\"dateInput\"\n" +
    "               ng-class=\"{true: 'selectedDateType', false: ''}[selectedDateType === 'end']\"/>\n" +
    "    </div>\n" +
    "    <div class=\"openButton\">\n" +
    "        <div ng-class=\"{true: 'Arrow-up', false: 'Arrow'}[calendarOpen]\"\n" +
    "             class=\"Arrow\"></div>\n" +
    "    </div>\n" +
    "\n" +
    "</div>\n" +
    "<div class=\"datePicker\" ng-if=\"calendarOpen\">\n" +
    "    <div class=\"monthYear\">\n" +
    "        <div class=\"arrow\" ng-click=\"prevMonth()\">\n" +
    "            <div class=\"LeftArrow\"></div>\n" +
    "        </div>\n" +
    "        <div class=\"thisMonth\"> {{showMonth}} {{showYear}}</div>\n" +
    "        <div class=\"arrow\" ng-click=\"nextMonth()\">\n" +
    "            <div class=\"RightArrow\"></div>\n" +
    "        </div>\n" +
    "    </div>\n" +
    "    <div class=\"days\">\n" +
    "        <table>\n" +
    "            <head>\n" +
    "                <th ng-class=\"daysHeader\" ng-repeat=\"header in dateGrid.headers\"> {{header.name}}</th>\n" +
    "            </head>\n" +
    "            <tr ng-repeat=\"week in dateGrid.weeks\">\n" +
    "                <th class=\"dayCell\" ng-class=\"day.class\" ng-repeat=\"day in week.days\" ng-click=\"selectDate(day)\">\n" +
    "                    {{day.name}}\n" +
    "                </th>\n" +
    "            </tr>\n" +
    "        </table>\n" +
    "\n" +
    "    </div>\n" +
    "    <div class=\"presets clearfix\">\n" +
    "        <div ng-repeat=\"button in settings.presets\" class=\"preset\">\n" +
    "            <div ng-click=\"selectPreset(button)\">\n" +
    "                {{button.name}}\n" +
    "            </div>\n" +
    "        </div>\n" +
    "    </div>\n" +
    "    <div class=\"buttons\">\n" +
    "        <div class=\"cancelButton\" ng-click=\"cancel()\">Cancel</div>\n" +
    "        <div class=\"applyButton\" ng-click=\"apply()\">Apply</div>\n" +
    "    </div>\n" +
    "</div>"
  );

}]);

})( window, window.angular );
