"use strict";

angular.module('glDatePickerController', [
  'glDatePickerFactory', 'vendor.jQuery'])
  .controller('glDatePickerController', ['$rootScope', '$scope', '$filter', 'glDatePickerFactory', '$',
    function ($rootScope, $scope, $filter, glDatePickerFactory, $) {

    var monthNames = ["January", "February", "March", "April", "May", "June",
      "July", "August", "September", "October", "November", "December"
    ];
    var weekDays = [{name: "M"}, {name: "T"}, {name: "W"}, {name: "T"}, {name: "F"}, {name: "S"}, {name: "S"}];
    var today = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      today.setHours(0, 0, 0);
    /**
     * @description initialize the calendar loading the preselected dates
     *
     */
    $scope.initDatePicker = function () {
      $scope.startDate = new Date($scope.dates[0]);
      $scope.endDate = new Date($scope.dates[1]);
      $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
      $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
      $scope.calendarOpen = false;
      $scope.selectedDateType = "";
      glDatePickerFactory.checkDefaultDates($scope);
      $scope.dateGrid = {
        headers: weekDays,
        weeks: []
      };
    };

    /**
     * @description opens up the calendar setting the date type to start or end
     *
     * @param {Object} dateType - the selected date type
     *
     */
    $scope.openCalendar = function (dateType) {
      if (!$scope.settings || !$scope.settings.disable) {
        $scope.selectedDateType = dateType;
        if ($scope.selectedDateType === "start") {
          $scope.baseDate = new Date($scope.startDate);
        } else if ($scope.selectedDateType === "end") {
          $scope.baseDate = new Date($scope.endDate);
        }
        $scope.baseDate.setDate(1);
        $scope.showMonth = monthNames[$scope.baseDate.getMonth()];
        $scope.showYear = $scope.baseDate.getFullYear();
        $scope.setDateGrid();
        $scope.calendarOpen = true;
      }
    };

    /**
     * @description loads the current page of the calendar
     *
     */
    $scope.setDateGrid = function () {
      $scope.dateGrid.weeks = glDatePickerFactory.setDateGrid($scope.baseDate, $scope.startDate, $scope.endDate, $scope.selectedDateType);
    };

    /**
     * @description sets the selected date as start or end date
     *
     * @param {Object} date - the selected date type
     *
     */
    $scope.selectDate = function (date) {
      if (date.isNotThisMonth === false) {
        $scope.selectedPreset = null;

        if ((glDatePickerFactory.compareDates(date.date, $scope.startDate) < 0) || ($scope.selectedDateType === "start" &&
          glDatePickerFactory.compareDates(date.date, $scope.endDate) <= 0 )) {
          $scope.startDate = date.date;
          $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
          $scope.selectedDateType = "end";
          $('#endInput').focus();
        } else if ((glDatePickerFactory.compareDates(date.date, $scope.endDate) > 0) || ($scope.selectedDateType === "end" &&
          glDatePickerFactory.compareDates(date.date, $scope.startDate) >= 0)) {
          $scope.endDate = date.date;
          $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
          $scope.selectedDateType = "start";
          $('#startInput').focus();
        }
        $scope.setDateGrid();
        glDatePickerFactory.checkDefaultDates($scope);
      }
    };

    /**
     * @description changes the selected month to next one
     *
     */
    $scope.nextMonth = function () {
      $scope.baseDate.setMonth($scope.baseDate.getMonth() + 1);
      $scope.setDateGrid();
      $scope.showMonth = monthNames[$scope.baseDate.getMonth()];
      $scope.showYear = $scope.baseDate.getFullYear();
    };

    /**
     * @description changes the selected month to previous one
     *
     */
    $scope.prevMonth = function () {
      $scope.baseDate.setMonth($scope.baseDate.getMonth() - 1);
      $scope.setDateGrid();
      $scope.showMonth = monthNames[$scope.baseDate.getMonth()];
      $scope.showYear = $scope.baseDate.getFullYear();
    };

    /**
     * @description restores last saved start and end date and close the calendar
     *
     */
    $scope.cancel = function () {
      $scope.calendarOpen = false;
      $scope.selectedDateType = "";
      $scope.startDate = new Date($scope.dates[0]);
      $scope.endDate = new Date($scope.dates[1]);
      $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
      $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
      if ($scope.relativeDates) {
        $scope.relativeStartDate = $scope.relativeDates[0];
        $scope.relativeEndDate = $scope.relativeDates[1];
        glDatePickerFactory.checkDefaultDates($scope);
      }
    };

    /**
     * @description set current start and end dates as selected one and closes the menu
     *
     */
    $scope.apply = function () {
      $scope.calendarOpen = false;
      $scope.selectedDateType = "";
      var setDates = {
        dates: [ $filter('date')($scope.startDate, 'yyyy-MM-dd'), $filter('date')($scope.endDate, 'yyyy-MM-dd')]
      };
      if ($scope.selectedPreset) {
        setDates.relativeDates = [$scope.relativeStartDate, $scope.relativeEndDate];
      } else {
        $scope.startDate.setHours(0, 0, 0);
        $scope.endDate.setHours(0, 0, 0);
        $scope.relativeStartDate = 'd-' + Math.ceil(Math.abs($scope.startDate.getTime() - today.getTime()) / (1000 * 3600 * 24));
        $scope.relativeEndDate = 'd-' + Math.ceil(Math.abs($scope.endDate.getTime() - today.getTime()) / (1000 * 3600 * 24));
        setDates.relativeDates = [$scope.relativeStartDate, $scope.relativeEndDate];
      }
      $rootScope.$broadcast('dateSelected', setDates);
    };

    /**
     * @description sets start and end dates according to the selected preset
     *
     * @param {Object} preset - the selected preset
     */
    $scope.selectPreset = function (preset) {
      $scope.selectedPreset = preset;
      $scope.startDate = glDatePickerFactory.getDateFromRelative(preset.startDate);
      $scope.endDate = glDatePickerFactory.getDateFromRelative(preset.endDate);
      $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
      $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
      $scope.relativeStartDate = preset.startDate;
      $scope.relativeEndDate = preset.endDate;

      $scope.openCalendar($scope.selectedDateType);
      glDatePickerFactory.checkDefaultDates($scope);
    };

    /**
     * @description checks if input dates are valid, if so sets start or end date otherwise rest input to default
     *
     * @param {string} dateType - start or end for start or end date
     */
    $scope.testDate = function (dateType) {
      var date = null;
      if (dateType === 'start') {
        date = new Date($scope.startDateDescription);
        if (date && !isNaN(date) && glDatePickerFactory.compareDates(date, $scope.endDate) <= 0
          && glDatePickerFactory.compareDates(date, $scope.startDate) !== 0) {
          $scope.startDate = new Date(date);
          $scope.openCalendar("end");
          $('#endInput').focus();
        } else {
          $scope.startDateDescription = $filter('date')($scope.startDate, 'yyyy-MM-dd');
        }
      } else if (dateType === 'end') {
        date = new Date($scope.endDateDescription);
        if (date && !isNaN(date) && glDatePickerFactory.compareDates(date, $scope.startDate) >= 0
          && glDatePickerFactory.compareDates(date, $scope.endDate) !== 0) {
          $scope.endDate = new Date(date);
          $scope.openCalendar("start");
          $('#startInput').focus();
        } else {
          $scope.endDateDescription = $filter('date')($scope.endDate, 'yyyy-MM-dd');
        }
      }
    };

    $scope.initDatePicker();

    $scope.$watchCollection('dates', function () {
      $scope.initDatePicker();
    });
}]);
