"use strict";

describe('datePicker controller', function() {
  beforeEach(module('glDatePickerController'));
  beforeEach(module('glDatePickerFactory'));

  var datePicker, $scope, $rootScope, controller, Filter, GlDatePickerFactory, event, eventData, document, element, $compile;


  beforeEach(inject( function (_$rootScope_, _$compile_, $controller, glDatePickerFactory, $filter) {
    $rootScope = _$rootScope_;
    GlDatePickerFactory = glDatePickerFactory;
    Filter = $filter;
    $compile = _$compile_;
    $scope = _$rootScope_.$new();
    $scope.dates = ['2015-07-20', '2015-07-28'];
    $scope.relativeDates = ['d7', 'd1'];

    element = angular.element('<input id="endInput" /> <input id="startInput" />');
    element = $compile(element)($scope);
    $scope.settings = {
        presets: [],
        defaults: ['d-1', 'd-7']
      }

    $rootScope.$on('dateSelected', function (ev, data) {event = 'dateSelected', eventData = data;});
    element = {
      focus: function () {return true;}
    };
    document = {
      getElementById: function (id) {return element;}
    };
    controller = $controller("glDatePickerController", {
      $scope: $scope,
      $element: element
    });
  }));

  describe('initDatePicker', function() {
    it('should call checkDefaultDates', function() {
      spyOn(GlDatePickerFactory, 'checkDefaultDates');
      $scope.initDatePicker();
      expect(GlDatePickerFactory.checkDefaultDates).toHaveBeenCalled();
    });
  });

  describe('openCalendar', function() {
    it('should should set baseDate month and year', function() {
      $scope.startDate = new Date($scope.dates[0]);
      $scope.endDate = new Date($scope.dates[1]);
      $scope.openCalendar("start");
      var date = new Date( $scope.startDate);
      date.setDate(1);
      expect(GlDatePickerFactory.compareDates($scope.baseDate, date)).toEqual(0);
      date = new Date( $scope.endDate);
      $scope.openCalendar("end");
      date.setDate(1);
      expect(GlDatePickerFactory.compareDates($scope.baseDate, date)).toEqual(0);
    });
  });

  describe('setDateGrid', function() {
    it('should call GlDatePickerFactory.setDateGrid', function() {
      spyOn(GlDatePickerFactory, 'setDateGrid');
      $scope.setDateGrid();
      expect(GlDatePickerFactory.setDateGrid).toHaveBeenCalled();
    });
  });

  describe('selectDate', function() {
    it('should set start date to the selected date', function() {
      $scope.selectedDateType = "start";
      $scope.baseDate = new Date(2015, 6, 1);
      var selDate = {date: new Date(2015, 6, 22), isNotThisMonth: false};
      $scope.selectDate(selDate);
      expect(GlDatePickerFactory.compareDates($scope.startDate, selDate.date)).toEqual(0);
    });

    it('should set end date to the selected date', function() {
      $scope.selectedDateType = "end";
      $scope.baseDate = new Date(2015, 6, 1);
      var selDate = {date: new Date(2015, 6, 26), isNotThisMonth: false};
      $scope.selectDate(selDate);
      expect($scope.endDate).toEqual(selDate.date);
    });
  });

  describe('nextMonth', function() {
    it('should change the month name showed and refresh days grid', function() {
      spyOn(GlDatePickerFactory, 'setDateGrid');
      $scope.baseDate = new Date(2015, 6, 1);
      $scope.nextMonth();
      expect($scope.showMonth).toEqual("August");
      expect(GlDatePickerFactory.setDateGrid).toHaveBeenCalled();
    });
  });

  describe('prevMonth', function() {
    it('should change the month name showed and refresh days grid', function() {
      spyOn(GlDatePickerFactory, 'setDateGrid');
      $scope.baseDate = new Date(2015, 6, 1);
      $scope.prevMonth();
      expect($scope.showMonth).toEqual("June");
      expect(GlDatePickerFactory.setDateGrid).toHaveBeenCalled();
    });
  });

  describe('cancel', function() {
    it('should close the calendar end set start and end date to original values', function() {
      spyOn(GlDatePickerFactory, 'checkDefaultDates');
      $scope.cancel();
      expect(GlDatePickerFactory.compareDates($scope.startDate, new Date(2015, 6, 20))).toEqual(0);
      expect(GlDatePickerFactory.compareDates($scope.endDate, new Date(2015, 6, 28))).toEqual(0);
      expect(GlDatePickerFactory.checkDefaultDates).toHaveBeenCalled();
    });
  });

  describe('select preset', function() {
    it('should set start date and end date according the relative values of the selected preset', function() {
     var preset = {
        startDate: 'w-1',
        endDate: 'we-1',
        name: "Last week"
      };
      $scope.baseDate = new Date();
      $scope.baseDate.setHours(0, 0, 0);
      $scope.baseDate.setDate(1);
      var presetStartDate = new Date();
      presetStartDate.setHours(0, 0, 0);
      var mon = (presetStartDate.getDay() === 0) ? 6 : presetStartDate.getDay() % 7 - 1;
      presetStartDate.setDate(presetStartDate.getDate() - (mon + 7));
      var presetEndDate = new Date(presetStartDate);
      presetEndDate.setDate(presetEndDate.getDate() + 6);
      $scope.selectPreset(preset);

      expect($scope.startDate.toString()).toEqual(presetStartDate.toString());
      expect($scope.endDate.toString()).toEqual(presetEndDate.toString());

    });
  });

  describe('testDate', function() {
    it('check if input start date is valid, if so change the start date accordingly', function() {
      $scope.baseDate = new Date(2015, 6, 1);
      $scope.startDateDescription = "2015-07-22";
      var date = new Date("2015-07-22");
      $scope.myForm = {$valid: true};
      $scope.testDate('start');
      expect(GlDatePickerFactory.compareDates($scope.startDate, date)).toEqual(0);
    });

    it('check if input start date is valid, if not reset the input date', function() {
      $scope.baseDate = new Date(2015, 6, 1);
      $scope.startDateDescription = "2015-07-uu";
      $scope.myForm = {$valid: false};
      $scope.testDate('start');
      expect($scope.startDateDescription).toEqual("2015-07-20");
    });

    it('check if input end date is valid, if so change the end date accordingly', function() {
      $scope.baseDate = new Date(2015, 6, 1);
      $scope.endDateDescription = "2015-07-27";
      var date = new Date("2015-07-27");
      $scope.myForm = {$valid: true};
      $scope.testDate('end');

      expect(GlDatePickerFactory.compareDates($scope.endDate, date)).toEqual(0);
    });

    it('check if input end date is valid, if not reset the input date', function() {
      $scope.baseDate = new Date(2015, 6, 1);
      $scope.endDateDescription = "2015-07-uu";
      $scope.myForm = {$valid: false};
      $scope.testDate('end');
      expect($scope.endDateDescription).toEqual("2015-07-28");
    });
  });

  describe('apply', function() {
    it('when a date was selected should change set start and end date and calculate the relative dates value', function() {
      $scope.startDate = new Date(2015, 6, 17);
      $scope.endDate = new Date(2015, 6, 24);
      $scope.apply();
      expect(eventData.dates).toEqual(['2015-07-17', '2015-07-24']);
    });
    it('when a preset was selected should change set start and end date and set relative dates values as described by the preset', function() {
      $scope.baseDate = new Date(2015, 6, 1);
      var preset = {
        startDate: 'w-1',
        endDate: 'we-1',
        name: "Last week"
      };
     var startDate = Filter('date')(GlDatePickerFactory.getDateFromRelative('w-1'), 'yyyy-MM-dd');
     var endDate = Filter('date')(GlDatePickerFactory.getDateFromRelative('we-1'), 'yyyy-MM-dd');
      $scope.selectPreset(preset);
      $scope.apply();
      expect(eventData.dates).toEqual([startDate, endDate]);
      expect(eventData.relativeDates).toEqual(['w-1', 'we-1']);
    });
  });
});